"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useStyle = void 0;
const ink_use_stdout_dimensions_1 = __importDefault(require("ink-use-stdout-dimensions"));
const react_1 = require("react");
const themes_1 = require("../themes");
/**
 * useStyle hook
 *
 */
const useStyle = (initialData = themes_1.defaultTheme) => {
    /**
     * Theme values
     */
    const [theme, setTheme] = (0, react_1.useState)(initialData);
    /**
     * Width and height of terminal viewport.
     */
    const [stdoutWidth, stdoutHeight] = (0, ink_use_stdout_dimensions_1.default)();
    const [width, setWidth] = (0, react_1.useState)(null);
    const [height, setHeight] = (0, react_1.useState)(null);
    /**
     * Active screen size
     */
    const [screen, setScreen] = (0, react_1.useState)(null);
    /**
     * Width of one column.
     */
    const [unit, setUnit] = (0, react_1.useState)(null);
    /**
     * Width and height of application.
     */
    const [bounds, setBounds] = (0, react_1.useState)({
        width,
        height,
    });
    (0, react_1.useEffect)(() => {
        typeof stdoutWidth === 'number' &&
            setWidth(stdoutWidth > 0 ? stdoutWidth : 70);
        typeof stdoutHeight === 'number' &&
            setHeight(stdoutHeight > 0 ? stdoutHeight : 25);
    }, [stdoutWidth, stdoutHeight]);
    /**
     * Set application based on viewport size.
     */
    (0, react_1.useEffect)(() => {
        setBounds({
            width: width > theme.maxWidth ? theme.maxWidth : width,
            height: height > theme.maxHeight ? theme.maxHeight : height,
        });
    }, [width, height, theme.maxHeight, theme.maxWidth]);
    /**
     * Set unit to be the total application width available
     * divided by the column count
     */
    (0, react_1.useEffect)(() => {
        setUnit(bounds.width / theme.columns);
    }, [bounds.width, theme.columns]);
    /**
     * Determine which screen size is currently active.
     */
    (0, react_1.useEffect)(() => {
        theme.screens.forEach(([lower, upper], iteration) => {
            bounds.width > lower && bounds.width < upper && setScreen(iteration);
        });
    }, [bounds, theme]);
    /**
     * Col
     * Function that returns the width for x columns
     */
    const col = (count) => {
        return unit * count;
    };
    /**
     * ctx
     *
     * Function that takes an array of possible display values
     * and returns the one that matches the current screen size.
     */
    const ctx = (screens) => {
        const value = screens[screen] ?? screens[screens.length - 1];
        return typeof value == 'number' ? Math.floor(value) : value;
    };
    /**
     * Set colors
     *
     * Merges colors onto theme.
     */
    const setColors = (colors) => {
        setTheme({
            ...theme,
            colors: {
                ...theme.colors,
                ...colors,
            },
        });
    };
    /**
     * Set screens
     *
     * Merges colors onto theme.
     */
    const setScreens = (screens) => {
        setTheme({
            ...theme,
            screens: [...theme.screens, ...screens],
        });
    };
    return {
        col,
        bounds,
        screen,
        ...theme,
        setColors,
        setScreens,
        ctx,
    };
};
exports.useStyle = useStyle;
